# Extracted from ./data/repos/black/src/black/concurrency.py
"""Run formatting of `sources` in parallel using the provided `executor`.

    (Use ProcessPoolExecutors for actual parallelism.)

    `write_back`, `fast`, and `mode` options are passed to
    :func:`format_file_in_place`.
    """
cache: Cache = {}
if write_back not in (WriteBack.DIFF, WriteBack.COLOR_DIFF):
    cache = read_cache(mode)
    sources, cached = filter_cached(cache, sources)
    for src in sorted(cached):
        report.done(src, Changed.CACHED)
if not sources:
    exit()

cancelled = []
sources_to_cache = []
lock = None
if write_back in (WriteBack.DIFF, WriteBack.COLOR_DIFF):
    # For diff output, we need locks to ensure we don't interleave output
    # from different processes.
    manager = Manager()
    lock = manager.Lock()
tasks = {
    asyncio.ensure_future(
        loop.run_in_executor(
            executor, format_file_in_place, src, fast, mode, write_back, lock
        )
    ): src
    for src in sorted(sources)
}
pending = tasks.keys()
try:
    loop.add_signal_handler(signal.SIGINT, cancel, pending)
    loop.add_signal_handler(signal.SIGTERM, cancel, pending)
except NotImplementedError:
    # There are no good alternatives for these on Windows.
    pass
while pending:
    done, _ = await asyncio.wait(pending, return_when=asyncio.FIRST_COMPLETED)
    for task in done:
        src = tasks.pop(task)
        if task.cancelled():
            cancelled.append(task)
        elif task.exception():
            report.failed(src, str(task.exception()))
        else:
            changed = Changed.YES if task.result() else Changed.NO
            # If the file was written back or was successfully checked as
            # well-formatted, store this information in the cache.
            if write_back is WriteBack.YES or (
                write_back is WriteBack.CHECK and changed is Changed.NO
            ):
                sources_to_cache.append(src)
            report.done(src, changed)
if cancelled:
    await asyncio.gather(*cancelled, return_exceptions=True)
if sources_to_cache:
    write_cache(cache, sources_to_cache, mode)
